#!/usr/bin/env python2
import optparse
import os.path
import subprocess
import sys
import syslog
import time


def log_info(msg):
    syslog.syslog(syslog.LOG_INFO | syslog.LOG_LOCAL1, msg)
    print(msg)


def update_process_name():
    sys.argv[0] = "synoame-notify-plugin.py"


def get_options_parser():
    parser = optparse.OptionParser()
    parser.add_option("--event", dest="event", help="{install|uninstall}")
    parser.add_option("--timing", dest="timing", help="{pre|post}")
    parser.add_option(
        "--package", dest="package", help="{CodecPack_HEVC_AAC,CodecPack_HEVC}"
    )
    parser.add_option("--version", dest="version", help="[pack version]")
    parser.add_option("--status", dest="status", help="{INSTALL,UNINSTALL,UPGRADE}")

    return parser


def validate_options(options):
    valid_options = [
        (options.event, ("install", "uninstall")),
        (options.timing, ("pre", "post")),
        (options.package, ("CodecPack_HEVC_AAC", "CodecPack_HEVC")),
        (options.status, ("INSTALL", "UNINSTALL", "UPGRADE")),
    ]
    for value, possible_values in valid_options:
        if value not in possible_values:
            return False
    return True


def run_plugin(plugin_path, parameter, env, message_prefix):
    before = time.time()

    p = subprocess.Popen([plugin_path, parameter], env=env,)
    p.communicate()

    after = time.time()
    time_diff = after - before

    log_info("{} ExitCode: {}".format(message_prefix, p.returncode))
    log_info("{} Runtime: {:0.3f} s".format(message_prefix, time_diff))


def run_plugins(event, timing, pack_name, pack_version, pack_status):
    parameter = "--{}".format(timing)
    full_event = "ame_pack_{}".format(event)
    plugin_dir = "/usr/local/libexec/{}".format(full_event)
    env = {
        "PACK_NAME": pack_name,
        "PACK_VERSION": pack_version,
        "PACK_STATUS": pack_status,
    }

    if not os.path.exists(plugin_dir):
        log_info("plugin dir {} not exists".format(plugin_dir))
        return

    log_info("iterate dir {}, event: {}, timing: {}, env: {}".format(plugin_dir, full_event, timing, env))
    for f in os.listdir(plugin_dir):
        plugin_path = os.path.join(plugin_dir, f)
        if not os.path.isfile(plugin_path):
            continue

        message_prefix = "[{}][{}][{}]".format(timing, full_event, f)
        try:
            run_plugin(plugin_path, parameter, env, message_prefix)
        except Exception as error:
            log_info("Failed to run {}, error: {}".format(plugin_path, error))


if __name__ == "__main__":
    update_process_name()

    parser = get_options_parser()
    (options, args) = parser.parse_args()

    if (
        not options.event
        or not options.timing
        or not options.package
        or not options.version
        or not options.status
    ):
        parser.print_help()
        sys.exit(1)

    if not validate_options(options):
        parser.print_help()
        sys.exit(1)

    try:
        run_plugins(
            options.event,
            options.timing,
            options.package,
            options.version,
            options.status,
        )
    except Exception as error:
        log_info("Failed to run plugins, error: {}".format(error))
        sys.exit(1)

    sys.exit(0)
